@@ -17,6 +17,7 @@
 #include <KLocalizedString>
 #include <QAccessible>
 #include <QAccessibleWidget>
+#include <QTextBoundaryFinder>
 
 /**
  * This class implements a QAccessible-interface for a KateViewInternal.
@@ -24,7 +25,7 @@
  * This is the root class for the kateview. The \a KateCursorAccessible class
  * represents the cursor in the kateview and is a child of this class.
  */
-class KateViewAccessible : public QAccessibleWidget, public QAccessibleTextInterface // FIXME maybe:, public QAccessibleEditableTextInterface
+class KateViewAccessible : public QAccessibleWidget, public QAccessibleTextInterface, public QAccessibleEditableTextInterface
 {
 public:
     explicit KateViewAccessible(KateViewInternal *view)
@@ -39,6 +40,8 @@ public:
 
     void *interface_cast(QAccessible::InterfaceType t) override
     {
+        if (t == QAccessible::EditableTextInterface)
+            return static_cast<QAccessibleEditableTextInterface *>(this);
         if (t == QAccessible::TextInterface)
             return static_cast<QAccessibleTextInterface *>(this);
         return nullptr;
@@ -109,6 +112,7 @@ public:
         *endOffset = characterCount();
         return QString();
     }
+
     QRect characterRect(int offset) const override
     {
         KTextEditor::Cursor c = cursorFromInt(offset);
@@ -120,15 +124,22 @@ public:
         QPoint size = view()->cursorToCoordinate(endCursor) - p;
         return QRect(view()->mapToGlobal(p), QSize(size.x(), size.y()));
     }
+
     int cursorPosition() const override
     {
         KTextEditor::Cursor c = view()->cursorPosition();
         return positionFromCursor(view(), c);
     }
-    int offsetAtPoint(const QPoint & /*point*/) const override
+
+    int offsetAtPoint(const QPoint &point) const override
     {
+        if (view()) {
+            KTextEditor::Cursor c = view()->coordinatesToCursor(point);
+            return positionFromCursor(view(), c);
+        }
         return 0;
     }
+
     void removeSelection(int selectionIndex) override
     {
         if (selectionIndex != 0) {
@@ -136,10 +147,16 @@ public:
         }
         view()->view()->clearSelection();
     }
-    void scrollToSubstring(int /*startIndex*/, int /*endIndex*/) override
+
+    void scrollToSubstring(int startIndex, int /*endIndex*/) override
     {
-        // FIXME
+        auto c = cursorFromInt(startIndex);
+        if (!c.isValid()) {
+            return;
+        }
+        view()->view()->setScrollPosition(c);
     }
+
     void selection(int selectionIndex, int *startOffset, int *endOffset) const override
     {
         if (selectionIndex != 0 || !view()->view()->selection()) {
@@ -156,10 +173,12 @@ public:
     {
         return view()->view()->selection() ? 1 : 0;
     }
+
     void setCursorPosition(int position) override
     {
         view()->view()->setCursorPosition(cursorFromInt(position));
     }
+
     void setSelection(int selectionIndex, int startOffset, int endOffset) override
     {
         if (selectionIndex != 0) {
@@ -168,6 +187,7 @@ public:
         KTextEditor::Range range = KTextEditor::Range(cursorFromInt(startOffset), cursorFromInt(endOffset));
         view()->view()->setSelection(range);
     }
+
     QString text(int startOffset, int endOffset) const override
     {
         if (startOffset > endOffset) {
@@ -176,27 +196,125 @@ public:
         return view()->view()->document()->text().mid(startOffset, endOffset - startOffset);
     }
 
+    QString textAtOffset(int offset, QAccessible::TextBoundaryType boundaryType, int *startOffset, int *endOffset) const override
+    {
+        *startOffset = -1;
+        *endOffset = -1;
+        if (!view() || !startOffset || !endOffset) {
+            return {};
+        }
+        if (offset == -1) {
+            offset = positionFromCursor(view(), view()->view()->doc()->documentEnd());
+        }
+        KTextEditor::Cursor c = cursorFromInt(offset);
+        if (!c.isValid()) {
+            return {};
+        }
+        auto doc = view()->view()->doc();
+
+        switch (boundaryType) {
+        case QAccessible::TextBoundaryType::CharBoundary: {
+            QString t = doc->characterAt(c);
+            *startOffset = offset;
+            *endOffset = *startOffset + 1;
+            return t;
+        } break;
+        case QAccessible::TextBoundaryType::WordBoundary: {
+            QString t = doc->wordAt(c);
+            *startOffset = offset;
+            *endOffset = offset + t.size();
+            return t;
+        } break;
+        case QAccessible::TextBoundaryType::LineBoundary:
+        case QAccessible::TextBoundaryType::ParagraphBoundary: {
+            const QString line = doc->line(c.line());
+            if (line.isEmpty()) {
+                *startOffset = offset;
+                *endOffset = offset;
+                return {};
+            }
+            const int start = positionFromCursor(view(), KTextEditor::Cursor(c.line(), 0));
+            *startOffset = start;
+            *endOffset = offset + line.size();
+            return line;
+        } break;
+        case QAccessible::TextBoundaryType::SentenceBoundary: {
+            const QString line = doc->line(c.line());
+            if (line.isEmpty()) {
+                *startOffset = offset;
+                *endOffset = offset;
+                return {};
+            }
+            QTextBoundaryFinder bf(QTextBoundaryFinder::BoundaryType::Sentence, line);
+            int e = bf.toNextBoundary();
+            if (e != -1) {
+                int start = positionFromCursor(view(), KTextEditor::Cursor(c.line(), 0));
+                *startOffset = start;
+                *endOffset = start + bf.position();
+                return line.mid(0, e);
+            }
+        } break;
+        case QAccessible::TextBoundaryType::NoBoundary: {
+            const QString text = doc->text();
+            *startOffset = 0;
+            *endOffset = text.size();
+            return text;
+        } break;
+        }
+
+        return {};
+    }
+
+    QString textBeforeOffset(int /*offset*/, QAccessible::TextBoundaryType /*boundaryType*/, int *startOffset, int *endOffset) const override
+    {
+        // FIXME
+        *startOffset = -1;
+        *endOffset = -1;
+        return {};
+    }
+    QString textAfterOffset(int /*offset*/, QAccessible::TextBoundaryType /*boundaryType*/, int *startOffset, int *endOffset) const override
+    {
+        // FIXME
+        *startOffset = -1;
+        *endOffset = -1;
+        return {};
+    }
+
+    void deleteText(int startOffset, int endOffset) override
+    {
+        KTextEditor::Document *document = view()->view()->document();
+        KTextEditor::Range range(document->offsetToCursor(startOffset), document->offsetToCursor(endOffset));
+        document->removeText(range);
+    }
+
+    void insertText(int offset, const QString &text) override
+    {
+        KTextEditor::Document *document = view()->view()->document();
+        KTextEditor::Cursor cursor = document->offsetToCursor(offset);
+        document->insertText(cursor, text);
+    }
+
+    void replaceText(int startOffset, int endOffset, const QString &text) override
+    {
+        KTextEditor::Document *document = view()->view()->document();
+        KTextEditor::Range range(document->offsetToCursor(startOffset), document->offsetToCursor(endOffset));
+        document->replaceText(range, text);
+    }
+
     /**
      * When possible, using the last returned value m_lastPosition do the count
      * from the last cursor position m_lastCursor.
      * @return the number of chars (including one character for new lines)
      *         from the beginning of the file.
      */
-    int positionFromCursor(KateViewInternal *view, const KTextEditor::Cursor &cursor) const
+    int positionFromCursor(KateViewInternal *view, KTextEditor::Cursor cursor) const
     {
         int pos = m_lastPosition;
         const KTextEditor::DocumentPrivate *doc = view->view()->doc();
 
         // m_lastPosition < 0 is invalid, calculate from the beginning of the document
         if (m_lastPosition < 0 || view != m_lastView) {
-            pos = 0;
-            // Default (worst) case
-            for (int line = 0; line < cursor.line(); ++line) {
-                pos += doc->lineLength(line);
-            }
-            // new line for each line
-            pos += cursor.line();
-            m_lastView = view;
+            pos = doc->cursorToOffset(cursor) - cursor.column();
         } else {
             // if the lines are the same, just add the cursor.column(), otherwise
             if (cursor.line() != m_lastCursor.line()) {
@@ -230,18 +348,7 @@ private:
 
     KTextEditor::Cursor cursorFromInt(int position) const
     {
-        int line = 0;
-        for (;;) {
-            const QString lineString = view()->view()->document()->line(line);
-            if (position > lineString.length()) {
-                // one is the newline
-                position -= lineString.length() + 1;
-                ++line;
-            } else {
-                break;
-            }
-        }
-        return KTextEditor::Cursor(line, position);
+        return view()->view()->doc()->offsetToCursor(position);
     }
 
     QString textLine(int shiftLines, int offset, int *startOffset, int *endOffset) const
